@@ -10,12 +10,14 @@ Revs:
 import corr, time, sys, numpy, os, logging
 
 def exit_fail():
-    print 'FAILURE DETECTED. Log entries:\n',lh.printMessages()
+    print 'FAILURE DETECTED. Log entries:\n',
+    c.log_handler.printMessages()
     print "Unexpected error:", sys.exc_info()
     try:
         c.disconnect_all()
     except: pass
-    #raise
+    if verbose:
+        raise
     exit()
 
 def exit_clean():
@@ -34,19 +36,23 @@ if __name__ == '__main__':
         help='Skip the initialision. ie Only monitor.')
     p.add_option('-r', '--n_retries', dest='n_retries', type='int', default=-1, 
         help='Number of times to try and sync the system before giving up. Set to -1 for infinity. Default: -1')
+    p.add_option('-v', '--verbose', dest='verbose',action='store_true', default=False, 
+        help='Be verbose about errors.')
+
 
     opts, args = p.parse_args(sys.argv[1:])
 
     if args==[]:
-        print 'Please specify a configuration file! \nExiting.'
-        exit()
+        config_file=None
+    else:
+        config_file=args[0]
+    verbose=opts.verbose
 
-lh=corr.log_handlers.DebugLogHandler()
-try:
-    print 'Loading the configuration file %s...'%args[0],
-    c=corr.corr_functions.Correlator(args[0],lh)
-    for logger in (c.floggers + c.xloggers): logger.setLevel(10)
-    print 'done.'
+try:    
+    print 'Connecting...',
+    c=corr.corr_functions.Correlator(config_file=config_file,log_level=logging.DEBUG if verbose else logging.INFO, connect=False)
+    c.connect()
+    print 'done'
 
     print('\nDeprogramming all FPGAs...'),
     sys.stdout.flush()
